@@ -22,9 +22,8 @@ We provide a script that imports the core functionality and defines helpful
 macros for writing safe code.
 
 @
-\$ curl --output dataframe \"https:\/\/raw.githubusercontent.com\/mchav\/dataframe\/refs\/heads\/main\/scripts\/dataframe.sh\"
-\$ chmod +x dataframe
-\$ export PATH=$PATH:$PWD/dataframe
+\$ cabal update
+\$ cabal install dataframe
 \$ dataframe
 Configuring library for fake-package-0...
 Warning: No exposed modules
@@ -115,6 +114,7 @@ __Exploration__
 
 __Row ops__
 
+  * @D.filter :: Expr a -> (a -> Bool) -> DataFrame -> DataFrame@
   * @D.filterWhere :: Expr Bool -> DataFrame -> DataFrame@
   * @D.sortBy :: SortOrder -> [Text] -> DataFrame -> DataFrame@
 
@@ -128,11 +128,11 @@ __Column ops__
 __Group & aggregate__
 
   * @D.groupBy :: [Text] -> DataFrame -> GroupedDataFrame@
-  * @D.aggregate :: [(Text, F.UExpr)] -> GroupedDataFrame -> DataFrame@
+  * @D.aggregate :: [NamedExpr] -> GroupedDataFrame -> DataFrame@
 
 __Joins__
 
-  * @D.innerJoin \/ D.leftJoin \/ D.rightJoin \/ D.fullJoin@
+  * @D.innerJoin \/ D.leftJoin \/ D.rightJoin \/ D.fullOuterJoin@
 
 == Expression DSL (F.*) at a glance
 Columns (typed):
@@ -216,7 +216,7 @@ module DataFrame (
     module Subset,
     module Transformations,
     module Aggregation,
-    module Sorting,
+    module Permutation,
     module Merge,
     module Join,
     module Statistics,
@@ -305,7 +305,11 @@ import DataFrame.Operations.Core as Core hiding (
  )
 import DataFrame.Operations.Join as Join
 import DataFrame.Operations.Merge as Merge
-import DataFrame.Operations.Sorting as Sorting
+import DataFrame.Operations.Permutation as Permutation (
+    SortOrder (..),
+    shuffle,
+    sortBy,
+ )
 import DataFrame.Operations.Statistics as Statistics (
     correlation,
     frequencies,
@@ -331,6 +335,7 @@ import DataFrame.Operations.Subset as Subset (
     exclude,
     filter,
     filterAllJust,
+    filterAllNothing,
     filterBy,
     filterJust,
     filterNothing,
