@@ -26,6 +26,7 @@ import Type.Reflection (typeRep)
 import DataFrame.Internal.Column (Column (..), isNumeric)
 import qualified DataFrame.Internal.Column as D
 import DataFrame.Internal.DataFrame (DataFrame (..), getColumn)
+import DataFrame.Internal.Expression
 import DataFrame.Operations.Core
 import qualified DataFrame.Operations.Subset as D
 import System.Directory
@@ -39,6 +40,7 @@ import System.Process (
     std_out,
     waitForProcess,
  )
+import Text.Printf
 
 newtype HtmlPlot = HtmlPlot T.Text deriving (Show)
 
@@ -94,18 +96,20 @@ wrapInHTML chartId content width height =
         , "px;height:"
         , T.pack (show height)
         , "px\"></canvas>\n"
-        , "<script src=\"https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.js\"></script>\n"
+        , "<script src=\"https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.6/require.min.js\"></script>\n"
         , "<script>\n"
+        , "setTimeout(() => {"
         , content
+        , "}, 200);"
         , "\n</script>\n"
         ]
 
 plotHistogram :: (HasCallStack) => T.Text -> DataFrame -> IO HtmlPlot
-plotHistogram colName = plotHistogramWith colName (defaultPlotConfig Histogram)
+plotHistogram colName = plotHistogramWith colName 30 (defaultPlotConfig Histogram)
 
 plotHistogramWith ::
-    (HasCallStack) => T.Text -> PlotConfig -> DataFrame -> IO HtmlPlot
-plotHistogramWith colName config df = do
+    (HasCallStack) => T.Text -> Int -> PlotConfig -> DataFrame -> IO HtmlPlot
+plotHistogramWith colName numBins config df = do
     chartId <- generateChartId
     let values = extractNumericColumn colName df
         (minVal, maxVal) = if null values then (0, 1) else (minimum values, maximum values)
@@ -115,7 +119,7 @@ plotHistogramWith colName config df = do
         counts = calculateHistogram values bins binWidth
 
         labels =
-            T.intercalate "," ["\"" <> T.pack (show (round b :: Int)) <> "\"" | b <- bins]
+            T.intercalate "," ["\"" <> T.pack (printf "%.2f" b) <> "\"" | b <- bins]
         dataPoints = T.intercalate "," [T.pack (show c) | c <- counts]
 
         chartTitle =
@@ -125,9 +129,11 @@ plotHistogramWith colName config df = do
 
         jsCode =
             T.concat
-                [ "new Chart(\""
+                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                , "var ctx = document.getElementById('"
                 , chartId
-                , "\", {\n"
+                , "').getContext('2d');\n"
+                , "new Chart(ctx , {\n"
                 , "  type: \"bar\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -153,7 +159,7 @@ plotHistogramWith colName config df = do
                 , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});"
+                , "});});"
                 ]
 
     return $
@@ -186,9 +192,11 @@ plotScatterWith xCol yCol config df = do
 
         jsCode =
             T.concat
-                [ "new Chart(\""
+                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                , "var ctx = document.getElementById('"
                 , chartId
-                , "\", {\n"
+                , "').getContext('2d');\n"
+                , "new Chart(ctx , {\n"
                 , "  type: \"scatter\",\n"
                 , "  data: {\n"
                 , "    datasets: [{\n"
@@ -215,7 +223,7 @@ plotScatterWith xCol yCol config df = do
                 , "\" } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});"
+                , "});});"
                 ]
 
     return $
@@ -246,7 +254,7 @@ plotScatterByWith xCol yCol grouping config df = do
                 ]
 
     datasets <- forM (zip uniqueVals colors) $ \(val, color) -> do
-        let filtered = D.filter grouping (== val) df'
+        let filtered = D.filter (Col grouping) (== val) df'
             xVals = extractNumericColumn xCol filtered
             yVals = extractNumericColumn yCol filtered
             points = zip xVals yVals
@@ -279,9 +287,11 @@ plotScatterByWith xCol yCol grouping config df = do
 
         jsCode =
             T.concat
-                [ "new Chart(\""
+                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                , "var ctx = document.getElementById('"
                 , chartId
-                , "\", {\n"
+                , "').getContext('2d');\n"
+                , "new Chart(ctx , {\n"
                 , "  type: \"scatter\",\n"
                 , "  data: {\n"
                 , "    datasets: [\n"
@@ -301,7 +311,7 @@ plotScatterByWith xCol yCol grouping config df = do
                 , "\" } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});"
+                , "});});"
                 ]
 
     return $
@@ -353,9 +363,11 @@ plotLinesWith xAxis colNames config df = do
 
         jsCode =
             T.concat
-                [ "new Chart(\""
+                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                , "var ctx = document.getElementById('"
                 , chartId
-                , "\", {\n"
+                , "').getContext('2d');\n"
+                , "new Chart(ctx , {\n"
                 , "  type: \"line\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -375,7 +387,7 @@ plotLinesWith xAxis colNames config df = do
                 , "\" } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});"
+                , "});});"
                 ]
 
     return $
@@ -407,9 +419,11 @@ plotSingleBars colName config df = do
 
                 jsCode =
                     T.concat
-                        [ "new Chart(\""
+                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                        , "var ctx = document.getElementById('"
                         , chartId
-                        , "\", {\n"
+                        , "').getContext('2d');\n"
+                        , "new Chart(ctx , {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -433,7 +447,7 @@ plotSingleBars colName config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});"
+                        , "});});"
                         ]
             return $
                 HtmlPlot $
@@ -451,8 +465,11 @@ plotSingleBars colName config df = do
 
                 jsCode =
                     T.concat
-                        [ "new Chart(\""
+                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                        , "var ctx = document.getElementById('"
                         , chartId
+                        , "').getContext('2d');\n"
+                        , "new Chart(ctx , {\n"
                         , "\", {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
@@ -477,7 +494,7 @@ plotSingleBars colName config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});"
+                        , "});});"
                         ]
             return $
                 HtmlPlot $
@@ -502,9 +519,11 @@ plotPieWith valCol labelCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "new Chart(\""
+                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                        , "var ctx = document.getElementById('"
                         , chartId
-                        , "\", {\n"
+                        , "').getContext('2d');\n"
+                        , "new Chart(ctx , {\n"
                         , "  type: \"pie\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -524,7 +543,7 @@ plotPieWith valCol labelCol config df = do
                         , chartTitle
                         , "\" }\n"
                         , "  }\n"
-                        , "});"
+                        , "});});"
                         ]
             return $
                 HtmlPlot $
@@ -546,9 +565,11 @@ plotPieWith valCol labelCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "new Chart(\""
+                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                        , "var ctx = document.getElementById('"
                         , chartId
-                        , "\", {\n"
+                        , "').getContext('2d');\n"
+                        , "new Chart(ctx , {\n"
                         , "  type: \"pie\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -568,7 +589,7 @@ plotPieWith valCol labelCol config df = do
                         , chartTitle
                         , "\" }\n"
                         , "  }\n"
-                        , "});"
+                        , "});});"
                         ]
             return $
                 HtmlPlot $
@@ -637,9 +658,11 @@ plotStackedBarsWith categoryCol valueColumns config df = do
 
         jsCode =
             T.concat
-                [ "new Chart(\""
+                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                , "var ctx = document.getElementById('"
                 , chartId
-                , "\", {\n"
+                , "').getContext('2d');\n"
+                , "new Chart(ctx , {\n"
                 , "  type: \"bar\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -658,7 +681,7 @@ plotStackedBarsWith categoryCol valueColumns config df = do
                 , "      yAxes: [{ stacked: true, ticks: { beginAtZero: true } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});"
+                , "});});"
                 ]
 
     return $
@@ -689,9 +712,11 @@ plotBoxPlotsWith colNames config df = do
 
         jsCode =
             T.concat
-                [ "new Chart(\""
+                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                , "var ctx = document.getElementById('"
                 , chartId
-                , "\", {\n"
+                , "').getContext('2d');\n"
+                , "new Chart(ctx , {\n"
                 , "  type: \"bar\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -715,7 +740,7 @@ plotBoxPlotsWith colNames config df = do
                 , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});"
+                , "});});"
                 ]
 
     return $
@@ -748,9 +773,11 @@ plotGroupedBarsWithN n groupCol valCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "new Chart(\""
+                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                        , "var ctx = document.getElementById('"
                         , chartId
-                        , "\", {\n"
+                        , "').getContext('2d');\n"
+                        , "new Chart(ctx , {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -776,7 +803,7 @@ plotGroupedBarsWithN n groupCol valCol config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});"
+                        , "});});"
                         ]
             return $
                 HtmlPlot $
@@ -800,9 +827,11 @@ plotGroupedBarsWithN n groupCol valCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "new Chart(\""
+                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
+                        , "var ctx = document.getElementById('"
                         , chartId
-                        , "\", {\n"
+                        , "').getContext('2d');\n"
+                        , "new Chart(ctx , {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -826,7 +855,7 @@ plotGroupedBarsWithN n groupCol valCol config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});"
+                        , "});});"
                         ]
             return $
                 HtmlPlot $
