@@ -334,6 +334,20 @@ atIndicesStable indexes (UnboxedColumn column) = UnboxedColumn $ VG.unsafeBackpe
 atIndicesStable indexes (OptionalColumn column) = OptionalColumn $ VG.unsafeBackpermute column (VG.convert indexes)
 {-# INLINE atIndicesStable #-}
 
+atIndicesWithNulls :: VB.Vector (Maybe Int) -> Column -> Column
+atIndicesWithNulls indices column
+    | VB.all isJust indices =
+        atIndicesStable
+            (VU.fromList (catMaybes (VB.toList indices)))
+            column
+    | otherwise = case column of
+        BoxedColumn col ->
+            OptionalColumn $ VB.map (fmap (col VB.!)) indices
+        UnboxedColumn col ->
+            OptionalColumn $ VB.map (fmap (col VU.!)) indices
+        OptionalColumn col ->
+            OptionalColumn $ VB.map (\ix -> ix >>= (col VB.!)) indices
+
 -- | Internal helper to get indices in a boxed vector.
 getIndices :: VU.Vector Int -> VB.Vector a -> VB.Vector a
 getIndices indices xs = VB.generate (VU.length indices) (\i -> xs VB.! (indices VU.! i))
@@ -1232,3 +1246,31 @@ toIntVector column =
                         , errorColumnName = Nothing
                         }
                     )
+
+toUnboxedVector ::
+    forall a.
+    (Columnable a, VU.Unbox a) => Column -> Either DataFrameException (VU.Vector a)
+toUnboxedVector column =
+    case column of
+        UnboxedColumn (f :: VU.Vector b) -> case testEquality (typeRep @a) (typeRep @b) of
+            Just Refl -> Right f
+            Nothing ->
+                Left $
+                    TypeMismatchException
+                        ( MkTypeErrorContext
+                            { userType = Right (typeRep @Int)
+                            , expectedType = Right (typeRep @a)
+                            , callingFunctionName = Just "toIntVector"
+                            , errorColumnName = Nothing
+                            }
+                        )
+        _ ->
+            Left $
+                TypeMismatchException
+                    ( MkTypeErrorContext
+                        { userType = Right (typeRep @a)
+                        , expectedType = Left (columnTypeString column) :: Either String (TypeRep ())
+                        , callingFunctionName = Just "toUnboxedVector"
+                        , errorColumnName = Nothing
+                        }
+                    )
